/*
 Author  : Loe Feijs and TU/e 2013.
 This example shows how to control a LED (LED via 470 Ohm between pin 9 and GND). 
 The LED goes on when the environment's bright-ness changes 
 suddenly. The averaged brightness is calculated by a moving 
 average with time constant of 20 seconds. The LDR is connected 
 between +5V and pin A0. The LDR has a pull-down resistor of 
 22k between pin A0 and GND. The LED fades out after 10 seconds. 
 It takes 2 seconds to fade out. 
 
 Author  : P.Peters
 Date    : 20131202
 
 Modified code such that it fits my test plug
 pin 9 is used as GND, and connects to the LED and button.
 pin 10 has a 470 Ohm resistor to the LED
 pin 11 has a 470 Ohm resistor to the other side of the button
 In order to operate, pin 9 must be 0, and pin 11 must have pullup
 Also added a fade in in the state machine when led switches on
 */

//FSM states:
#define OFF 0
#define ON  1
#define FADINGOUT 2
#define FADINGIN 3

//connect LED, LDR:
#define gndPin 9
#define ledPin 6
#define buttonPin 11
#define ldrPin A0

#define onTime 400 //times 10 milliseconds
#define fadeTime 200 //times 10 milliseconds

int state = OFF;
int timer = 0;
boolean LDRevent = false;

void LEDstep(){
  switch (state){
  case OFF: 
    //    if (digitalRead(buttonPin) == LOW) {
    if (LDRevent){
      // prepare timer for delay
      timer = fadeTime;
      // switch state
      state = FADINGIN;
    } 
    break;
  case ON: 
    if (timer > 0){
      timer--;
    } 
    else {
      // prepare timer for fading out
      timer = fadeTime;
      // switch state
      state = FADINGOUT;
    } 
    break;
  case FADINGOUT:
    if (timer > 0){
      //fade from 250 to 0 during fadeTime steps
      timer--;
      analogWrite(ledPin,10*timer/8); 
    }  
    else {
      // switch state
      state = OFF;
      LDRevent = false;
      // state transition actions
      analogWrite(ledPin,0);
    }
    break;
  case FADINGIN:
    if (timer > 0){
      //fade from 0 to 250 during fadeInTime steps
      timer--;
      analogWrite(ledPin,250-10*timer/8); 
    }  
    else {
      // switch state
      state = ON;
      timer = onTime;
      // state transition actions
      analogWrite(ledPin,255);
    }
    break;
  }
}

float avg;  

void setup()  { 
  pinMode(gndPin,OUTPUT);
  digitalWrite(gndPin,0);
  pinMode(ledPin, OUTPUT); 
  pinMode(buttonPin, INPUT_PULLUP);
  pinMode(ldrPin, INPUT);
  avg = (float)analogRead(ldrPin);
} 

void LDRstep(){
  //LDR from +5V to pin A0, pulldown 22k to GND
  int v = analogRead(ldrPin);
  // next average takes approx 20 second to even out 10ms * (1 / 0.0005)
  avg = 0.9995 * avg + 0.0005* (float)v;
  float delta = 20.0; // 20 times 10msec 
  float dif = (float)v - avg;
  if (abs(dif) > delta)
    LDRevent = true;
}

void loop()  { 
  LDRstep();
  LEDstep();
  delay(10); // 10 ms delay between steps
}







